
/* LN MTP code */
proc nlmixed data=move;
	
	/* define bounds and starting values */
	bounds 0 < sigma2; * sigma2=sigma**2;
	parms a0=1 a1=0 a2=0 a3=0 a4=0
		b0=1 b1=0 b2=0 b3=0 b4=0 sigma2=1;

	/* binary part specification */
	linbin = a0 + a1*move + a2*bmi + a3*age + a4*dcg;
	binprob = exp(linbin)/(1+exp(linbin)); /* prob y > 0 */

	/* MTP specification of location parameter of the LN distribution */
	mu = b0 + b1*move + b2*bmi + b3*age + b4*dcg - log(binprob) - sigma2/2;

	/* log likelihood for zero costs */
	if total_cost=0 then loglik=log(1-binprob);
	
	/* log likelihood for positive costs */
	else if total_cost>0 then loglik=log(binprob)-log(total_cost)
		-.5*log(2*CONSTANT('PI'))-log(sqrt(sigma2))-(1/(2*sigma2))*(log(total_cost)-mu)**2;

	/* fit the model */
	model total_cost~general(loglik);

	/* output predictions and estimated marginal means */
	totalcostpred=exp(b0+b1*move+b2*bmi+b3*age+b4*dcg);
	predict totalcostpred out=mtpln_pred;
	estimate '1st quartile age & bmi, move enrollee' exp(b0+b1+b2*32+b3*55+b4*0.11);
	estimate '1st quartile age & bmi, move non-enrollee' exp(b0+b2*32+b3*55+b4*0.11);
	estimate 'median age & bmi, move enrollee' exp(b0+b1+b2*34+b3*61+b4*0.22);
	estimate 'median age & bmi, move non-enrollee' exp(b0+b2*34+b3*61+b4*0.22);
	estimate '3rd quartile age & bmi, move enrollee' exp(b0+b1+b2*37+b3*66+b4*0.32);
	estimate '3rd quartile age & bmi, move non-enrollee' exp(b0+b2*37+b3*66+b4*0.32);
run;


/* LSN MTP code */
proc nlmixed data=move;

	/* define bounds and starting values */
	bounds 0<sigma;
	parms a0=1 a1=0 a2=0 a3=0 a4=0  
		b0=1 b1=0 b2=0 b3=0 b4=0 sigma=1 kappa=-1; 

	/* binary part specification */
	linbin = a0 + a1*move + a2*bmi + a3*age + a4*dcg;
	binprob = exp(linbin)/(1+exp(linbin)); /* prob y > 0 */

	/* MTP specification of location parameter of the LSN distribution */
	delta = kappa/sqrt(1+kappa**2);
	mu = b0 + b1*move + b2*bmi + b3*age + b4*dcg - log(2) - log(binprob) - log(CDF('NORMAL',sigma*delta, 0, 1))-sigma**2/2;

	/* log likelihood for zero costs */
	if total_cost=0 then loglik=log(1-binprob);

	/* log likelihood for positive costs */
	else if total_cost>0 then do;
		pdfnormvar=(log(total_cost)-mu)/sigma;
		cdfnormvar=kappa*((log(total_cost)-mu)/sigma);
		loglik=log(binprob)+log(2)-log(total_cost)-log(sigma)+log(PDF('NORMAL', pdfnormvar, 0, 1)) + 
		log(CDF('NORMAL', cdfnormvar, 0,1));
	end;

	/* fit the model */
	model total_cost~general(loglik);

	/* output predictions and estimated marginal means */
	totalcostpred=exp(b0+b1*move+b2*bmi+b3*age+b4*dcg);
	predict totalcostpred out=mtplsn_pred;
	estimate '1st quartile age & bmi, move enrollee' exp(b0+b1+b2*32+b3*55+b4*0.11);
	estimate '1st quartile age & bmi, move non-enrollee' exp(b0+b2*32+b3*55+b4*0.11);
	estimate 'median age & bmi, move enrollee' exp(b0+b1+b2*34+b3*61+b4*0.22);
	estimate 'median age & bmi, move non-enrollee' exp(b0+b2*34+b3*61+b4*0.22);
	estimate '3rd quartile age & bmi, move enrollee' exp(b0+b1+b2*37+b3*66+b4*0.32);
	estimate '3rd quartile age & bmi, move non-enrollee' exp(b0+b2*37+b3*66+b4*0.32);
run;


/* GG MTP code */
proc nlmixed data=move;

	/* define bounds and starting values */
	bounds 0<sigma;
	parms a0=1 a1=0 a2=0 a3=0 a4=0
		b0=1 b1=0 b2=0 b3=0 b4=0 sigma=2 kappa=-.1;

	/* binary part specification */
	linbin = a0 + a1*move + a2*bmi + a3*age + a4*dcg;
	binprob = exp(linbin)/(1+exp(linbin)); /* prob y > 0 */
	
	/* MTP specification of location parameter of the GG distribution */
	mu = b0 + b1*move + b2*bmi + b3*age + b4*dcg - log(binprob) - (sigma*log((kappa)**2))/kappa-log(GAMMA(1/((kappa)**2)
			+sigma/kappa))+log(GAMMA(1/((kappa)**2)));
	eta=abs(kappa)**(-2);

	/* log likelihood for zero costs */
	if total_cost=0 then loglik=log(1-binprob);

	/* log likelihood for positive costs */
	else if total_cost>0 then do;
		u = SIGN(kappa)*(log(total_cost)-mu)/sigma;
		loglik=log(binprob)+eta*log(eta)-log(sigma)-log(total_cost)-log(GAMMA(eta))-.5*log(eta) + u*sqrt(eta)-eta*exp(abs(kappa)*u);
	end;

	/* fit the model */
	model total_cost~general(loglik);
	
	/* output predictions and estimated marginal means */
	totalcostpred=exp(b0+b1*move+b2*bmi+b3*age+b4*dcg);
	predict totalcostpred out=mtpgg_pred;
	estimate '1st quartile age & bmi, move enrollee' exp(b0+b1+b2*32+b3*55+b4*0.11);
	estimate '1st quartile age & bmi, move non-enrollee' exp(b0+b2*32+b3*55+b4*0.11);
	estimate 'median age & bmi, move enrollee' exp(b0+b1+b2*34+b3*61+b4*0.22);
	estimate 'median age & bmi, move non-enrollee' exp(b0+b2*34+b3*61+b4*0.22);
	estimate '3rd quartile age & bmi, move enrollee' exp(b0+b1+b2*37+b3*66+b4*0.32);
	estimate '3rd quartile age & bmi, move non-enrollee' exp(b0+b2*37+b3*66+b4*0.32);
run;
