*********************************************
* Case Study 1.sas							*
* Fit Various ZI Models	for Case Study 1 	*
* Infille: "I:/Brian/Two Part Chapter/Access Files/access_data.txt";
* NOTE: CIs from estimate statement in NLMIXED    *
*		may differ from exponentiating by hand,   *
*		so may not exactly match results in paper *
* May 25, 2015								*
*********************************************;
option nodate nonumber nocenter;
data access;
 infile  "I:/Brian/Two Part Chapter/Access Files/access_data.txt" firstobs=2;
 input gender subab visit y race id;
 male=(gender=1);
 visit3=(visit=3);
 visit12=(visit=12);
run;
proc print data=access (obs=5); run;

*ods pdf file="I:/Brian/Two Part Chapter/Access Files/Case1_Model_Output.pdf"; 

* Model 1: Random Effects Poisson;
ods select FitStatistics;
proc nlmixed data=access;
   parameters b0=0 b1=0 b2=0 s2u=1;
    eta  = b0+b1*visit3+b2*visit12+u;
    mu   = exp(eta);
    model y ~ poisson(mu);
	random u ~ normal(0,s2u) subject=id; 
run;

* Model 2: Random Effects NB;
ods select FitStatistics AdditionalEstimates;
proc nlmixed data=access;
    parameters b0=0 b1=0 b2=0 r=1 s2u=1;
    eta  = b0+b1*visit3+b2*visit12+u;
    mu   = exp(eta);
	p    = mu/(r+mu); * Prob of a "success";
	q    = 1-p; 	  * Prob of "failure" (note, this is how nlmixed negbin function is parameterized;
	model y~negbin(r,q);	
	random u ~ normal(0,s2u) subject=id; 
	estimate "theta" 1/r;
run;

* Model 3: Random effects ZIP;
ods select FitStatistics;
proc nlmixed data=access;
   parameters b0=0 b1=0 b2=0 s1=1 s12=0 s2=1
              a0=0 a1=0 a2=0 ;
   * Binary part;
   	eta1 = a0+a1*visit3+a2*visit12+u1;
	expeta1=exp(eta1);
   	phi  = expeta1/(1+expeta1);
   * Count part;
    eta2  = b0+b1*visit3+b2*visit12+u2;
    mu   = exp(eta2);
   /* Build the ZIP log likelihood */
    if y=0 then 
         ll = log((1-phi) + phi*exp(-mu));
    else ll = log(phi) - mu + y*log(mu) - lgamma(y + 1);
    model y ~ general(ll);
	random u1 u2 ~ normal([0,0],[s1,s12,s2]) subject=id; 
run;

* Model 4: Random effects ZINB;
*ods html file="I:/Brian/Two Part Chapter/Access Files/ZINB_Estimates.xls";
*ods tagsets.simplelatex file="I:/Brian/Two Part Chapter/Access Files/ZINB_Estimates.tex";
ods select FitStatistics ParameterEstimates AdditionalEstimates;
proc nlmixed data=access;
   parameters b0=0 b1=0 b2=0 r=1 s1=1 s2=1 rho=0
              a0=0 a1=0 a2=0 ;
   * Binary part;
   	eta1 = a0+a1*visit3+a2*visit12+u1;
	expeta1=exp(eta1);
   	phi  = expeta1/(1+expeta1);
   * Count part;
    eta2 = b0+b1*visit3+b2*visit12+u2;
    mu   = exp(eta2);
	p    = mu/(r+mu); * Prob of a success;
   /* Build the ZINB log likelihood */
    if y=0 then 
         ll = log((1-phi) + phi*((1-p)**r));
    else ll = log(phi) + lgamma(r+y) - lgamma(y+1) -lgamma(r) + r*log(1-p) + y*log(p);  * L = Gamma(r+y)/[Gamma(y+1)*Gamma(r)] (1-p)^r p^y;
    model y ~ general(ll);
	random u1 u2 ~ normal([0,0],[s1**2,rho*s1*s2,s2**2]) subject=id; 
	estimate "alpha (1/r)" 1/r;
	estimate "exp(a1)" exp(a1); /* NOTE: CIS MAY DIFFER SLIGHTLY THAN THOSE OBTAINED BY EXPONENTIATING ORIGINAL CIS -- MAY NOT EXACTLY MATCH PAPER */
	estimate "exp(a2)" exp(a2);
	estimate "exp(b1)" exp(b1);
	estimate "exp(b2)" exp(b2);
	estimate "mean visits at 3 months" exp(b0+b1);
	estimate "exp(b2)" exp(b2);
	estimate "rho" rho;
	estimate "s1**2" s1**2;
	estimate "s2**2" s2**2;
	estimate "Overall mean, 3 Months" exp(a0+a1)/(1+exp(a0+a1))*exp(b0+b1); * Overall marginal mean at 3-month follow-up;
	estimate "phi_3month RE 2SD above" exp(a0+a1+2*s1)/(1+exp(a0+a1+2*s1));
	estimate "phi_12month RE 2SD above" exp(a0+a2+2*s1)/(1+exp(a0+a2+2*s1));
	estimate "phi_3month RE 2SD below" exp(a0+a1-2*s1)/(1+exp(a0+a1-2*s1));
	estimate "phi_12month RE 2SD below" exp(a0+a2-2*s1)/(1+exp(a0+a2-2*s1));
run;
*ods ods tagsets.simplelatex close;
*ods html close;
ods pdf close;
